////////////////////////////////////////////////////////////////////////////////
//
// A class implementing a deme within a population, exchanging migrants with
// other demes
//
//
////////////////////////////////////////////////////////////////////////////////


#include "deme.h"
 
#include "cond_var.h"

#ifdef _SHOW_MUT_DISTR_
	#include <fstream.h>
	ofstream f_site_hits("mut_hits_distr.sum");
	static count_simul=0;
#endif

//------------------------------------------------------------------------------

//Initialization of static members of TDeme
TMigrationMatrix* TDeme::Migrations=NULL;

//==============================================================================
//------------------------------------------------------------------------------
TDeme&
TDeme::operator=(const TDeme& D) {
	_growth_rate=D._growth_rate;
   _sample_size=D._sample_size;
   _deme_size=D._deme_size;
   _lineages=D._lineages;
   _lidum=D._lidum;
   _id=D._id;
   _min_deme_size=D._min_deme_size;
   _max_deme_size=D._max_deme_size;
   _proba_coal=D._proba_coal;
   Migrations=D.Migrations;
   _is_migration=D._is_migration;
   NodeList.de_allocate_list();
   int size=D.NodeList.ListSize;
   if (size) {                
   	NodeList.allocate_list(size);
   	for (int i=0; i<size; ++i) {
      	NodeList[i]=D.NodeList[i];
      }
   }
   NodeList.de_allocate_list();
   size=D.OriginalNodes.ListSize;
   if (size) {
   	OriginalNodes.allocate_list(size);
   	for (int i=0; i<size; ++i) {
      	OriginalNodes[i]=D.OriginalNodes[i];
      }
   }
   
   _polym_sites=D._polym_sites;
   return *this;
}
//------------------------------------------------------------------------------
int
TDeme::copy_node_pointers(const TTree& T, const int curr_lin, const long& max_lin) {

   int base=curr_lin;

   //This node list is dynamic and will change according to migrations and coalescences
   NodeList.allocate_list(max_lin);

   //Keeps a copy of the original nodes
   OriginalNodes.allocate_list(_sample_size);

	for (int i=0; i<_sample_size; ++i, ++base) {
   	//Stamp the node as originally belonging to the current deme
   	T(base)->set_deme(_id);
      //Then copy the node pointer to the local lists
   	NodeList[i]=T(base);
   	OriginalNodes[i]=T(base);
   }
   //Filling up the node list once for all with empty pointers
   //so that no further size change is required during the generation
   //of the genealogy
   for (int i=_sample_size; i<max_lin; ++i) {
   	NodeList[i]=NULL;
   }
   return 1;
};
//------------------------------------------------------------------------------
//Check and implements migrations
int          
TDeme::send_migrants(TDeme& sink, const int& i) {
   if (!_lineages) return 0;
   //We will allow for multiple migrations per generation
   //Loro: !!! A faster routine could be made by first getting the number of migrants
   //from a binomial distribution
   for (int j=0, num_lineages=_lineages; j<num_lineages; ++j) {
   	if (ran3(&_lidum)<=(*Migrations)(_id,i)) migrate(sink);
   }
   return 1;
};
//------------------------------------------------------------------------------
//Implements the coalescence process within population
int
TDeme::coalesce_lineages(const long& time, TTree& GeneTree, const int& pos) {

   //If there is only one lineage left, do not even try to coalesce it...
   if (_lineages<2) return 0;

	if (!_deme_size) {
      cout 	<< "TDeme::coalesce_lineages() : deme size of population "
      		<< _id << " is zero !" << endl;
   	return 0;
   }

   double proba_coal=_proba_coal*0.5*_lineages*(_lineages-1);

   if (proba_coal==0.0) return 0;

   //Coalesce two lineages with probability proba
   if (ran3(&_lidum)<=proba_coal) {
   	//Pick two lineages at random for a coalescence
      int pick1= (int) (ran3(&_lidum)*_lineages), pick2= (int) (ran3(&_lidum)*_lineages);
      long count=0;
      while (pick1==pick2) {
      	pick2=(int) (ran3(&_lidum)*_lineages);
         if (++count>1000) return 0;
      }
      
      TNode *desc1=NodeList[pick1],
      		*desc2=NodeList[pick2],
            *ancestor=GeneTree(pos);

      //Update node information
      ancestor->desc1=desc1;
      ancestor->desc2=desc2;
      ancestor->time=time;
      desc1->ancestor=ancestor;
      desc2->ancestor=ancestor;

		//Replace one coalescing node by the ancestor
      //and swap the second with the last node after updating the number
      //of remaining lineages 
      NodeList[pick1]=ancestor;
      NodeList[pick2]=NodeList[--_lineages];
      return 1;
   }
   return 0;
};

//------------------------------------------------------------------------------
void
TDeme::reset() {

   //Restoring initial number of lineages
	_lineages=_sample_size;

   int size=NodeList.size();
   //Copying original nodes into dynamic list
	for (int i=0; i<_sample_size; ++i) {
   	NodeList[i]=OriginalNodes[i];
   }
   //Filling up the node list once for all with empty pointers
   for (int i=_sample_size; i<size; ++i) {
   	NodeList[i]=NULL;
   }
   _min_deme_size=MIN_SIZE;
   _max_deme_size=MAX_SIZE;
   _proba_coal=1.0/_deme_size;

}
//------------------------------------------------------------------------------
//A simple procedure to check whether there are migrations from this population
int
TDeme::check_migrations(){
   if (Migrations) {
      _is_migration=false;
   	int size=Migrations->size();
      for (int i=0; i<size; ++i)
      if (_id!=i) {
      	if ((*Migrations)(_id,i)>0.0) {
         	_is_migration=true;
            return 1;
         }
      }
      return 0;
   }
   return 0;
};
//------------------------------------------------------------------------------
void
TDeme::print_haplotypes_for_Arlequin(ostream & os, const int& n, const Mut_Type data_type) {

   char DNA_letters[4]={'A','G','C','T'};
   int len=OriginalNodes[0]->seq_length;

   os << "#Number of mutation hits per site\n"
   	<< "#Sites ";
   for (int j=0; j<len; ++j) {
      	os
            << setw(4)
            << setiosflags(ios::right)
         	<< j << " ";
      }      
   os << "\n";

   #ifdef _SHOW_MUT_DISTR_
   	f_site_hits << "Simulation #" << ++count_simul << "\t";
   #endif

   os << "#Hits  ";
   for (int j=0; j<len; ++j) {
      	os 
            << setw(4)
            << setiosflags(ios::right)
            << OriginalNodes[0]->hits[j] << " ";

         #ifdef _SHOW_MUT_DISTR_
      	f_site_hits
            << setiosflags(ios::right)
            << OriginalNodes[0]->hits[j] << "\t";
         #endif
      }
   os << "\n";

   #ifdef _SHOW_MUT_DISTR_
   f_site_hits  << "\n";
   #endif

   for (int i=0; i<_sample_size; ++i) {
   	//Write id and frequency
   	os << n << "_" << i << " 1 ";
      TNode* CurNode=OriginalNodes[i];
      if (CurNode->sequence) {
      	int* cursite=CurNode->sequence->begin();
      	for (int j=0; j<len; ++j) {
         	if (data_type==DNA)
   					//os << DNA_letters[(*CurNode->sequence)[j]];
   					os << DNA_letters[*cursite++];
               else
   					//os << (*CurNode->sequence)[j];
                  //Loro_1_3_99 : In order to avoid negative numbers
               	if (data_type==MICROSAT) os << (10000+*cursite++) << " ";
               	else os << *cursite++;
         }
         os << "\n";
      }
   }
};
//------------------------------------------------------------------------------
void
TDeme::print_haplotypes_for_PAUP(ostream & os, const Mut_Type data_type) {

   char DNA_letters[4]={'A','G','C','T'};
   int len=OriginalNodes[0]->seq_length;

   for (int i=0; i<_sample_size; ++i) {
   	TNode* CurNode=OriginalNodes[i];
      if (CurNode->sequence) {    
      	int* cursite=CurNode->sequence->begin();
			//Write id
      	os << CurNode->node_number << "." << (CurNode->deme+1) << "\n";
      	for (int j=0; j<len; ++j) {
         	if (data_type==DNA)
   					//os << DNA_letters[(*CurNode->sequence)[j]];
   					os << DNA_letters[*cursite++];
               else
   					//os << (*CurNode->sequence)[j];
                  //Loro_1_3_99 : In order to avoid negative numbers
               	if (data_type==MICROSAT) os << (10000+*cursite++) << " ";
               	else os << *cursite++;
         }
         os << "\n";
      }
   }
};
//------------------------------------------------------------------------------
ostream&
operator<<(ostream& os , const TDeme& D) {
	os  	<< "\nDeme size   : " << D._deme_size
   		<< "\nSample size : " << D._sample_size
         << "\nGrowth rate : " << D._growth_rate
         << "\n";
   return os;
}

//------------------------------------------------------------------------------
ostream&
operator<<(ostream& os, const THistoricalEvent& HE) {
	os	<< "\nTime             : " << HE.time
   	<< "\nSource           : " << HE.source
      << "\nSink             : " << HE.sink
      << "\nMigrants         : " << HE.migrants
      << "\nNew size         : " << HE.new_deme_size
      << "\nNew growth rate  : " << HE.new_growth_rate
      << "\nNew migr. matrix : " << HE.MigMat
      << endl;
   return os;
}
//------------------------------------------------------------------------------
istream&
operator>>(istream& is, THistoricalEvent& HE) {
	is	>> HE.time
   	>> HE.source
      >> HE.sink
      >> HE.migrants
      >> HE.new_deme_size
      >> HE.new_growth_rate
      >> HE.MigMat;
   return is;
}
//------------------------------------------------------------------------------
//==============================================================================
//------------------------------------------------------------------------------
long
TDemeCollection::count_lineages() {
   long count=0;
   for (int i=0 ;i<_num_demes; ++i) {
   	count+=Demes[i].lineages();
   }
   return count;
};
//------------------------------------------------------------------------------
 TDemeCollection&
 TDemeCollection::operator=(const TDemeCollection& DC) {
   _is_migration=DC._is_migration;
   _num_demes=DC._num_demes;
   _cur_mig_mat=DC._cur_mig_mat;
   if (Demes) delete[] Demes;
   if (DC.Demes) {
      Demes= new TDeme[DC._num_demes];
   	for (int i=0; i<DC._num_demes; ++i) {
      	Demes[i]=DC.Demes[i];;
      }
   }
   Events.Flush();
   if (DC.Events.GetItemsInContainer()) {
   	for (int i=0; i<DC.Events.GetItemsInContainer(); ++i) {
      	Events.Add(DC.Events[i]);
      }
   }
   MigrMatArray=DC.MigrMatArray;
   GeneTree=DC.GeneTree;
   if (CoalTimes) {
   	delete CoalTimes;
      CoalTimes=NULL;
   }
   if (DC.CoalTimes) {
      	try {
         	CoalTimes= new TMigrationMatrix();
         }
         catch (...) {
         	cout 	<< "TDeme::operator=(const TDeme& D): unable to allocate memory"
                  << endl;
            if (CoalTimes) {
      			delete CoalTimes;
      			CoalTimes=NULL;
      		}
            return *this;
         }
   	*CoalTimes=*DC.CoalTimes;
   }
   if (PairDiff) {
   	delete PairDiff;
      PairDiff=NULL;
   }
   if (DC.PairDiff) {
      	try {
         	PairDiff= new TMigrationMatrix();
         }
         catch (...) {
         	cout 	<< "TDeme::operator=(const TDeme& D): unable to allocate memory"
                  << endl;
            if (PairDiff) {
      			delete PairDiff;
      			PairDiff=NULL;
      		}
            return *this;
         }
   	*PairDiff=*DC.PairDiff;
   }
   if (MinCoalTimes) {
   	delete MinCoalTimes;
      MinCoalTimes=NULL;
   }
   if (DC.MinCoalTimes) {
      	try {
         	MinCoalTimes= new TMigrationMatrix();
         }
         catch (...) {
         	cout 	<< "TDeme::operator=(const TDeme& D): unable to allocate memory"
                  << endl;
   			if (MinCoalTimes) {
      			delete MinCoalTimes;
      			MinCoalTimes=NULL;
      		}
            return *this;
         }
   	*MinCoalTimes=*DC.MinCoalTimes;
   }
   _polym_sites=DC._polym_sites;
   _tot_mut=DC._tot_mut;
   return *this;
}
//------------------------------------------------------------------------------
//Check for migrations and update coalescence probabilities within each population
int
TDemeCollection::check_migrations(){
   _is_migration=false;
   //Examine all populations in turns to know which
   //one experience migrations. !! Do not stop the checking after first migration found
   for (int i=0; i<_num_demes; ++i)  {
   	if (Demes[i].check_migrations()) {
      	_is_migration=true;
      }
   }
   if (_is_migration) return 1;
   return 0;
};
//------------------------------------------------------------------------------
int
TDemeCollection::check_growth(){
   for (int i=0; i<_num_demes; ++i)  {
   	if (Demes[i].growth()!=0.0) {
         return 1;
      }
   }
   return 0;
};
//------------------------------------------------------------------------------
int
TDemeCollection::create_demes(const TMigrMatArray* const MA) {


	MigrMatArray=(TMigrMatArray*)MA;
   
	int size=(*MA)[0].size(); //Get number of demes from migration matrix

   TDeme aDeme;

   //Copies the pointer to the migration matrix in each deme
   aDeme.set_migration_matrix(&(*MigrMatArray)[0]);

   if (_num_demes!=size) {
   	if (Demes) delete[] Demes;
      _num_demes=size;
      Demes= new TDeme[_num_demes];
   }

   for (int i=0; i<size; ++i) {
   	//Set different ids for each deme
   	aDeme.set_id(i);
   	Demes[i]=aDeme;
   }
   return _num_demes;
};
//------------------------------------------------------------------------------
int
TDemeCollection::create_lineages(const TSampSizeArray& SS) {

	initialize_samp_size(SS);

   int size=SS.GetItemsInContainer();
   if (size!=_num_demes) return 0;

   //Count total initial number of lineages (sum of sample sizes)
   long num_lineages=count_lineages();

   //Allocate room for the tree structure
   if (num_lineages) GeneTree.allocate_tree(num_lineages);
   else {
   	cout << "TDemeCollection::create_lineages() : Tree is empty " << endl;
      return 0;
   }

   //Copy the pointers to the nodes of the genealogy in each deme
	for (int i=0, current_lineage=0; i<size; ++i) {
   	Demes[i].copy_node_pointers(GeneTree,current_lineage,num_lineages);
      current_lineage+=Demes[i].lineages();
   }
 	return 1;
};
//------------------------------------------------------------------------------
int
TDemeCollection::initialize_growth_rates(const TGrowthArray& GR) {
	int size=GR.GetItemsInContainer(); //Get number of demes from migration matrix
   if (size!=_num_demes) return 0;
  for (int i=0; i<size; ++i) {
   	Demes[i].set_growth_rate(GR[i]);
//      cout << i << "\t" << Demes[i].growth() << "\n";
   }
   return 1;
};
//------------------------------------------------------------------------------
int
TDemeCollection::initialize_deme_size(const TDemeSizeArray& DS) {
	int size=DS.GetItemsInContainer();
   if (size!=_num_demes) return 0;
   for (int i=0; i<size; ++i) {
   	Demes[i].set_deme_size(DS[i]);
   }
   return 1;
};
//------------------------------------------------------------------------------
int
TDemeCollection::initialize_samp_size(const TSampSizeArray& SS) {
	int size=SS.GetItemsInContainer();
   if (size!=_num_demes) return 0;
   for (int i=0; i<size; ++i) {
   	Demes[i].set_sample_size(SS[i]);
   }
   return 1;
};
//------------------------------------------------------------------------------
int
TDemeCollection::initialize_events(const TEventArray& HE) {
	int size=HE.GetItemsInContainer();
   for (int i=0; i<size; ++i) {
   	Events.Add(HE[i]);
      cout << Events[i] << endl;
   }
   return 1;
};
//------------------------------------------------------------------------------
int
TDemeCollection::implement_event(const int& cur_event) {
   THistoricalEvent& CurEvent=Events[cur_event];
   //Step 1: Send migrants from source to sink
   // Check also that the sink population has a deme size >0
   if (CurEvent.migrants>0.0 && Demes[CurEvent.sink].deme_size()) {
   //int num_migrants= (int) (Demes[CurEvent.source].lineages()*CurEvent.migrants);
   //Loro_07_06_04 BBBB UUUUU GGGGG BUG BUG BUG BUG
        int numLineages=Demes[CurEvent.source].lineages();
        int num_migrants=0;
        long lidum=1;
        for (int i=0; i<numLineages; ++i) {
                if (ran3(&lidum)<CurEvent.migrants) ++num_migrants;
        }
        for (int i=0; i<num_migrants; ++i) {
                Demes[CurEvent.source].migrate(Demes[CurEvent.sink]);
        }
   }
   //Step 2: Resize sink deme size
   Demes[CurEvent.sink].linear_resize(CurEvent.new_deme_size);
   //Step 3: Readjust sink growth rate
   Demes[CurEvent.sink].set_growth_rate(CurEvent.new_growth_rate);
   //Step 4: Set new migration matrix
   if (_cur_mig_mat!=CurEvent.MigMat) {
   	TDeme aDeme;
      //Copies the static pointer to the migration matrix in each deme
   	aDeme.set_migration_matrix(&(*MigrMatArray)[CurEvent.MigMat]);
      _cur_mig_mat=CurEvent.MigMat;
   }
   return 1;
};
//------------------------------------------------------------------------------
int
TDemeCollection::build_tree() {

   long num_lineages=count_lineages();
   long pos=2*GeneTree.sample_size()-num_lineages;
   int next_event=0, num_event=Events.GetItemsInContainer();

   //Loro_21_9_99 :
   //By default take the first migration matrix of the array: zero migration matrix
   // if no migration matrix is provided
   TDeme aDeme;
   //Copies the static pointer to the migration matrix in each deme
   aDeme.set_migration_matrix(&(*MigrMatArray)[0]);
   _cur_mig_mat=0;

   //Check that there are migrations in the genealogical process
   bool 	implement_migration=check_migrations(),
   		implement_growth=check_growth();

   //We'll simulate a maximum of MAX_GENERATIONS generations
   for (long time=0, j, k; time<MAX_GENERATIONS && num_lineages>1; ++time) {

      //Step 1: Check for possible historical events
      if (next_event<num_event) {
      	while (Events[next_event].time==time) {
         	implement_event(next_event);
         	//Check that the migration matrix has not changed
         	implement_migration=check_migrations();
         	//Check that the growth rates have not changed
         	implement_growth=check_growth();
            //Loro_21_9_99: To prevent the comparison of an unexisting event !  
         	if (++next_event==num_event) break;
         }
      }

		//Step 2: Migration round
      if (implement_migration) {
      	for (j=0; j<_num_demes; ++j) {
         	if (Demes[j].is_migration()) {
      			for (k=0; k<_num_demes; ++k) {
         			if (j!=k) Demes[j].send_migrants(Demes[k],k);
               }
         	}
      	}
      }

   	//Step 3: Coalescence round
      for (j=0; j<_num_demes; ++j) {
      	if (Demes[j].coalesce_lineages(time, GeneTree, pos)) {
         	--num_lineages;
            ++pos;
         }
      }

   	//Step 4: Resize deme size after growth for next generation
      if (implement_growth)
      for (j=0; j<_num_demes; ++j) {
      	if (Demes[j].growth()) {
            if (!Demes[j].exponential_resize()) {
            	cout 	<< "TDemeCollection::build_tree(): size of deme "
               		<< j << "is zero !" << endl;
               return 0;
            }
         }
      }
   }
   if (num_lineages>1) {
   	cout 	<< "TDemeCollection:: build_tree(): Coalescence process did not converge"
      		<< endl;
      return 0;
   }
   return 1;
};
//------------------------------------------------------------------------------
void
TDemeCollection::reset(const TDemeSizeArray& SA, const TGrowthArray& GA) {

   initialize_deme_size(SA);
   initialize_growth_rates(GA);

   for (int i=0; i<_num_demes; ++i) {
   	Demes[i].reset();
   }
};
//------------------------------------------------------------------------------
int
TDemeCollection::compute_moments_of_demes_coalescence_times() {

   //Get the root node
	TNode& root=GeneTree[2*GeneTree.sample_size()-2];

   //Build the list of descendent nodes
   root.build_lists_of_descendent_nodes();

   if (!CoalTimes) {
   	try {
   		CoalTimes=new TMigrationMatrix(_num_demes,ISLAND,0.0);
   	}
      catch (...) {
      	if (CoalTimes) delete CoalTimes; CoalTimes=NULL;
         cout  << "TDemeCollection::compute_moments_of_demes_coalescence_times()"
         		<< ": unable to allocate memory"
               << endl;
         return 0;
      }
   }
   else CoalTimes->set_elems(0.0);

  	if (!PairDiff) {
   	try {
   		PairDiff=new TMigrationMatrix(_num_demes,ISLAND,0.0);
   	}
      catch (...) {
      	if (PairDiff) delete PairDiff; PairDiff=NULL;
         cout  << "TDemeCollection::compute_moments_of_demes_coalescence_times()"
         		<< ": unable to allocate memory"
               << endl;
         return 0;
      }
   }
   else PairDiff->set_elems(0.0);

   if (!MinCoalTimes) {
   	try {
   		MinCoalTimes=new TMigrationMatrix(_num_demes,ISLAND,0.0);
   	}
      catch (...) {
      	if (MinCoalTimes) delete MinCoalTimes; MinCoalTimes=NULL;
         cout  << "TDemeCollection::compute_moments_of_demes_coalescence_times()"
         		<< ": unable to allocate memory"
               << endl;
         return 0;
      }
   }
   else MinCoalTimes->set_elems(MAXLONG);

   //Compute total coalescent times within and among demes
   root.compute_total_coal_times_among_demes(*this,*CoalTimes, *PairDiff, *MinCoalTimes);

   //Compute mean coalescent times
   for (int i=0; i<_num_demes; ++i) {
      long sampsize1=Demes[i].sample_size();
   	//Loro_20_9_99
      if (sampsize1)
      //Among pop treatment
   	for (int j=0; j<i; ++j) {     
      	long sampsize2=Demes[j].sample_size();
      	if (sampsize2) {
      		(*CoalTimes)(i,j)/=sampsize1*Demes[j].sample_size();
      		(*PairDiff)(i,j)/=sampsize1*Demes[j].sample_size();
         }
      }
      //Within pop treatment
      if (sampsize1>1L) {
      	(*CoalTimes)(i,i)/=sampsize1*(sampsize1-1)*0.5;
      	(*PairDiff)(i,i)/=sampsize1*(sampsize1-1)*0.5;
      }
   }
	return 1;
};
//------------------------------------------------------------------------------
//A procedure to sprinkle mutations once the tree branch lengths are known
const long&
TDemeCollection::sprinkle_mutations(const float& mut_rate, const int& len,
						const Mut_Type& mut_type, const float& gamma_par,
                  const float& mut_ratio, const float& prop_sites,
                  const float& trans_rate,
                  const int& range_constr) {
   //Get the root node
	TNode& root=GeneTree[2*GeneTree.sample_size()-2];

	//Set tree length to zero
   root.reset_tree_length();

   //Initialize hits per site to zero
   int size=root.hits.GetItemsInContainer(), to_add=len-size;
   //Loro_29_8_98 //Resize sequence length if needed
	for (int i=0; i<to_add; ++i){
         root.hits.Add(0);
   }
   for (int i=0; i<len; ++i)
   		root.hits[i]=0;
   /*
   if (!size) {
   	for (int i=0; i<len; ++i){
         root.hits.Add(0);
      }
   }
   else {
   	if (size==len) {
      	for (int i=0; i<root.seq_length; ++i)
   		root.hits[i]=0;
      }
      else {
      	cout << "<nTDemeCollection::sprinkle_mutations: hits vector has a bad size \n";
         _tot_mut=0;
         return _tot_mut;
      }
   }
	*/
	/*
   //Create the vector of gamma distributed mutation rates once for all
   if (!gamma_par==0.0) {

   	if (!root.mut_rates.size()) {
      	cout << "\nInitializing unequal mutation rates...";
      	root.mut_rates.build(len,gamma_par, mut_ratio,prop_sites);
         cout << "done\n\n";
      }
      //Loro_27_8_98: Mutation rates are not update any more between simulations
   	//else root.mut_rates.get_new_rates(len,gamma_par, mut_ratio, prop_sites);
   }
	*/
   long my_lidum=1L; //Any positive number will not reset the random number generator
   //Call a recursive procedure to add mutation according to a given mutation rate
   //per generation
   root.set_mut_rate(mut_rate);
   _tot_mut=root.add_mutations(&my_lidum, 0, len,
   									 mut_type, gamma_par, trans_rate, range_constr);
   return _tot_mut;
}
//------------------------------------------------------------------------------
extern ostream& operator<<(ostream& os,const TDemeCollection& DC) {

	for (int i=0, numdemes=DC.num_demes(); i<numdemes; ++i) {
   	os << "\nDeme #" << (i+1)
      	<< "\n----------\n\n"
         << DC.Demes[i]
         << "\n---------------------------------------\n";
   }

   int size=DC.MigrMatArray->GetItemsInContainer();
   for (int i=0; i<size; ++i) {
   	os << "\nMIGRATION MATRIX " << i
   		<< "\n=================\n"
      	<< (*DC.MigrMatArray)[i]
      	<< "\n";
   }

   os << "\nHISTORICAL EVENTS"
   	<< "\n=================\n";

   for (int i=0, numevents=DC.Events.GetItemsInContainer(); i<numevents; ++i) {
   	os << "\nEvent #" << (i+1)
      	<< "\n----------\n\n"
         << DC.Events[i]
         << "\n---------------------------------------\n";
	}
   return os;
};

//------------------------------------------------------------------------------
void
TDemeCollection::write_samples_to_Arlequin_file(ostream& os, const Mut_Type data_type) {
   for (int i=0; i<_num_demes; ++i)
   if (Demes[i].sample_size())  //Loro_20_9_99
   {
   	write_Arlequin_sample_header(i+1, Demes[i].sample_size(), os);
      Demes[i].print_haplotypes_for_Arlequin(os, i, data_type);
      write_Arlequin_sample_footer(os);
   }

}
//------------------------------------------------------------------------------
void
TDemeCollection::write_group_section_to_Arlequin_file(ostream& os) {
	os
   	<< "\n[[Structure]]\n"
      << "\n\tStructureName=\"Simulated data\""
      << "\n\tNbGroups=1"
      << "\n\tGroup={";
   for (int i=0; i<_num_demes; ++i) {
   	if (Demes[i].sample_size())   //Loro_20_9_99
   	os
      	<< "\n\t   \"Sample " << (i+1) << "\"";
   }
   os
   	<< "\n\t}" << endl;
};
//------------------------------------------------------------------------------
void
TDemeCollection::write_samples_to_PAUP_file(ostream& os, const Mut_Type data_type) {
   for (int i=0; i<_num_demes; ++i) {
		Demes[i].print_haplotypes_for_PAUP(os, data_type);
   }
}
//------------------------------------------------------------------------------

