#if !defined(genealogy_h)              // Sentry, use file only if it's not already included.
#define genealogy_h

#include <fstream.h>
#include <stdlib.h>
#include "arrays.h"
#include "mutation.h"

enum tree_output_type {GENERATIONS=0, MUT_RATE=1, NUM_MUT=2};

extern float UNIT_TIME;

typedef MY_TArrayAsVector<int> TIntVect;

//Declaration of two classes defined elsewhere but needed here
class TDemeCollection;
class TMigrationMatrix;

class TNode {
	public:   
   	long time;
      TNode *desc1;
      TNode *desc2;
      TNode *ancestor;
      TIntVect* desc1_nodes;
      TIntVect* desc2_nodes;
      TIntVect* sequence;     //The sequence to mutate
      static TIntVect hits;   //The number of hits per site
      static GammaRates mut_rates; //A static vector of gamma distributed rates
      int seq_length;
      static double mut_rate;			//Global mutation rate along the tree
      long num_new_mut;       //Number of mutations as compared to the direct ancestor
      long left_mut;
      long right_mut;
      int node_number;
      int deme;         		//Remember to which deme it belongs
      static double tree_length; //Length of the tree to which the node belongs, expressed in number of mutations
      static int min_mic, max_mic; //Range constraints for microsatellite data //Loro_26_2_99
   	TNode() {
			time=0;
         desc1=desc2=ancestor=NULL;
         desc1_nodes=desc2_nodes=NULL;
         num_desc1=num_desc2=0;
         node_count++;
         node_number=node_count;
         deme=-1;
         num_new_mut=left_mut=right_mut=0;
         seq_length=0;
         sequence=NULL;
      }      
      TNode(const TNode& N)   {
      	if (this!=&N) *this=N;
      }
      ~TNode() {
      	if (desc1_nodes) {
            desc1_nodes->Flush();
         	delete desc1_nodes;
            desc1_nodes=NULL;
         }
   		if (desc2_nodes) {
            desc2_nodes->Flush();
         	delete desc2_nodes;
            desc2_nodes=NULL;
         }
   		if (sequence) {
            sequence->Flush();
         	delete sequence;
            sequence=NULL;
         }
      }
      int operator==(const TNode& N) {
      	if ( 	desc1==N.desc1  	&&
         		desc2==N.desc2		&&
               ancestor==N.ancestor)  return 1;
         return 0;
      }
      TNode& operator=(const TNode& N);
      
      void set_deme(const int d) {deme=d;}
      void set_mut_rate(const double& m) {mut_rate=m;}
      double tree_exp_length() {return tree_length;}
      void reset_tree_length() {tree_length=0;}
      int count_desc();
      int count_polym_sites();
      
      int build_lists_of_descendent_nodes();
      int compute_moments_of_pairwise_divergence(double& sum_time, double& sum_square_time);
   	int compute_total_coal_times_among_demes(	TDemeCollection& DemeCollec,
      														TMigrationMatrix& CoalTimes,
      														TMigrationMatrix& PairDiff,
   													  		TMigrationMatrix& MinCoalTimes);
      /*void print_desc_nodes(TDrawingBoard& DB, int node_posx, int node_posy,
      								char node, char hor_bar, char left_corner,
                              char right_corner, char vert_bar);
      */
      int print_info(ostream& os);
   	int num_desc1, num_desc2;
      void reset_node_count() {node_count=0;};
      long add_mutations(long *lidum, const int& num_mut,
      		const int& len, const Mut_Type& mut_type, const float& gamma_par,
            const float& trans_rate, const int& range_const);
      //A recursive routine to print tree topology and branch length
      void print_tree_structure(ostream& os, const tree_output_type tree_type, const float & mu);

   protected:
   	static int node_count;
};

ostream & operator<<(ostream& os, const TNode& node);

//An array of pointers to Nodes
//It is like an indirect array of Nodes where Nodes would not be deleted
//in the destructor of the indirect array
class TListNode {
	public:
		TListNode() {list=NULL; ListSize=0;}
		TListNode(const int size, TNode * tree);
		~TListNode() {
			if (list) delete[] list;
      }
      TNode *& operator[](const int n) const {return list[n];}
      int ListSize;
      int allocate_list(const int size);
      void de_allocate_list() {
      	if (list) delete[] list;
         list=NULL;
      };
      const int& size() {return _size;}
	private:
   	TNode **list;
      int _size;
};

class TTree {
	public:
   	TTree() {
      	SampleSize=0;
      	tree=NULL;
         delete_nodes=1;
         _mean_coalescence_time=0;
         _sd_coalescence_time=0;
      };
   	TTree(int size);
   	~TTree() {
      	if (tree) {
      		if (delete_nodes) delete[] tree;
         	else delete tree;
         }
      }
		TNode& operator[](const int n) const {return tree[n];}
      TNode* operator()(const int i) const {return tree + i;}
  		int allocate_tree(const int& sampsize);
      int build_tree();
      int bottleneck(double time, double factor);
      void print_nodes(ostream& os, const tree_output_type tree_type, const float & mu);
      void print_sequences(ostream& os, const Mut_Type& mut_type);
      int sample_size() const {return SampleSize;}
      void set_delete_nodes() {delete_nodes=1;}
      void set_not_delete_nodes() {delete_nodes=0;}
      float total_length() {if (tree) return tree[0].tree_length; else return 0;};
      int compute_moments_of_coalescence_times();
      const double& mean_coalescence_time() {return _mean_coalescence_time;}
      const double& sd_coalescence_time() {return _sd_coalescence_time;}

   private:
   	int SampleSize;
   	TNode* tree;
      int delete_nodes;
      double _mean_coalescence_time;
      double _sd_coalescence_time;
      double _tree_subst_length;
      double _num_mut_on_tree;
};

#endif  // genealogy_h sentry.

